==> 2.5.1.tar.gz: lib/src/node.dart <==
import 'dart:async';
import 'dart:io';
import 'crypto.dart';
import 'packet.dart';
import 'net_addr.dart';
import 'peer_info.dart';
import 'tail_io.dart';
import 'utils.dart';
import 'verival.dart';
import 'io.dart';

class Node {
  final List<int> sk;
  final Tail tail;
  final File? cache;
  bool _closed = false;

  Node(this.sk, this.tail, [this.cache]) {
    listen();
  }

  static Future<Node> create(List<int> sk, NetAddr tail, [File? cache]) async {
    return Node(await Crypto.genSK(), await Tail.connect(tail), cache);
  }

  void close() {
    if (_closed) {
      return;
    }
    tail.close();
    _closed = true;
  }

  bool get closed {
    return _closed;
  }

  Future<bool> _setLastPacket(Packet packet) async {
    var last = await _getLastPacket();
    if (last is Packet && last.id > packet.id) {
      return false;
    }
    _lastPacket = packet;
    if (cache != null) {
      await cache?.writeAsBytes(packet.encode());
    }
    return true;
  }

  Packet? _lastPacket;

  Future<Packet?> _getLastPacket() async {
    try {
      return Packet.decode(await cache!.readAsBytes());
    } catch (error) {
      return _lastPacket;
    }
  }

  Stream<Input> get stream {
    return tail.stream
        .asyncMap((dg) async {
          try {
            var packet = Packet.decode(dg);
            if (!Utils.equal(packet.to, pk())) {
              return null;
            }
            await _setLastPacket(packet);
            return packet;
          } catch (_) {
            return null;
          }
        })
        .where((packet) => packet is Packet)
        .asyncMap((packet) async {
          try {
            return await Input.decrypt(packet!, sk);
          } catch (_) {
            return null;
          }
        })
        .where((msg) => msg is Input)
        .map((msg) => msg!);
  }

  Duration delay = Duration(milliseconds: 50);

  List<int> pk() {
    return Crypto.derivePK(sk);
  }

  void listen() {
    Timer.run(() async {
      for (; !_closed;) {
        try {
          await Future.delayed(delay);
          // delay = Duration(milliseconds: 500);
          var last = await _getLastPacket();
          last ??= Packet(pk(),
              List.filled(pk().length, 0), [], 1);
          await tail.check(
              Packet(last.to, List.filled(last.to.length, 0), [], last.id - 1)
                  .encode());
          await tail.check(last.encode());
        } catch (_) {}
      }
    });
  }

  Future<void> publish(String name) {
    return Verival.publish(PeerInfo(tail: tail.host, name: name), sk);
  }

  Future<void> send(Output out) async {
    var packet = await out.encrypt(sk);
    var encoded = packet.encode();
    if (encoded.length > 400) {
      throw Exception("too large packet");
    }
    var peerInfo = await Verival.lookup(packet.to);
    await Tail.push(peerInfo.tail, encoded, sk);
  }
}

==> 2.6.0.tar.gz: lib/src/node.dart <==
import 'dart:async';
import 'mem_cache.dart';

import 'crypto.dart';
import 'packet.dart';
import 'net_addr.dart';
import 'peer_info.dart';
import 'tail_io.dart';
import 'utils.dart';
import 'verival.dart';
import 'io.dart';
import 'cache.dart';

class Node {
  final List<int> sk;
  final Tail tail;
  final Cache cache;
  bool _closed = false;

  Node(this.sk, this.tail, this.cache) {
    listen();
  }

  static Future<Node> create(List<int> sk, NetAddr tail, [Cache? cache]) async {
    return Node(
        sk, await Tail.connect(tail), cache is Cache ? cache : MemCache());
  }

  void close() {
    if (_closed) {
      return;
    }
    tail.close();
    _closed = true;
  }

  bool get closed {
    return _closed;
  }

  Future<bool> _setLastPacket(Packet packet) async {
    var last = await _getLastPacket();
    if (last is Packet && last.id > packet.id) {
      return false;
    }
    await cache.set("last_packet", packet.encode());
    return true;
  }

  Future<Packet?> _getLastPacket() async {
    try {
      return Packet.decode((await cache.get("last_packet"))!);
    } catch (error) {
      return null;
    }
  }

  Stream<Input> get stream {
    return tail.stream
        .asyncMap((dg) async {
          try {
            var packet = Packet.decode(dg);
            if (!Utils.equal(packet.to, pk())) {
              return null;
            }
            await _setLastPacket(packet);
            return packet;
          } catch (_) {
            return null;
          }
        })
        .where((packet) => packet is Packet)
        .asyncMap((packet) async {
          try {
            return await Input.decrypt(packet!, sk);
          } catch (_) {
            return null;
          }
        })
        .where((msg) => msg is Input)
        .map((msg) => msg!);
  }

  Duration delay = Duration(milliseconds: 50);

  List<int> pk() {
    return Crypto.derivePK(sk);
  }

  void listen() {
    Timer.run(() async {
      for (; !_closed;) {
        try {
          await Future.delayed(delay);
          // delay = Duration(milliseconds: 500);
          var last = await _getLastPacket();
          last ??= Packet(pk(), List.filled(pk().length, 0), [], 1);
          await tail.check(
              Packet(last.to, List.filled(last.to.length, 0), [], last.id - 1)
                  .encode());
          await tail.check(last.encode());
        } catch (_) {}
      }
    });
  }

  Future<void> publish(String name) {
    return Verival.publish(PeerInfo(tail: tail.host, name: name), sk);
  }

  Future<void> send(Output out) async {
    var packet = await out.encrypt(sk);
    var encoded = packet.encode();
    if (encoded.length > 400) {
      throw Exception("too large packet");
    }
    var peerInfo = await Verival.lookup(packet.to);
    await Tail.push(peerInfo.tail, encoded, sk);
  }
}
